/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Highlander.
 *
 * The Initial Developer of the Original Code is
 *
 * Einar Egilsson. (email: highlander@einaregilsson.com)
 *
 * Portions created by the Initial Developer are Copyright (C) 2006 - 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Malte Kraus <firefox@maltekraus.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var HighlanderWhitelist = {

  list : [],

  init : function() {
    this.load();
    this.prefObserver.register();
  },

  save : function() {
    SeanConnery.setCharPref('whitelist', this.list.toSource());
  },

  load : function() {
    this.list = eval(SeanConnery.getCharPref('whitelist'));
  },

  add : function(pattern) {
    this.list.push(pattern);
    this.save();
  },

  updateAt : function(index, pattern) {
    this.list[index] = pattern;
    this.save();
  },

  deleteAt : function(index) {
    this.list.splice(index, 1);
    this.save();
  },

  isWhitelisted : function(url) {
    for each (var pattern in this.list) {
      if (this.isWildcardMatch(pattern, url)) {
        SeanConnery.debug("%1 is whitelisted by pattern %2"._(url, pattern));  
        return true;
      }
    }
    return false;
  },

  isWildcardMatch : function(pattern, url) {
    var parts
    , part
    , i
    , pos;

    parts = pattern.split('*');

    for (i in parts) {

      part = parts[i];

      pos = url.indexOf(part);

      if (pos == -1) {
        return false;
      }

      if (i == 0 && pos != 0) {
        return false;
      }

      if (i == parts.length -1 && i != "" && url.substr(url.length - part.length) != part) {
        return false;
      }

      url = url.substr(pos + part.length);
    }

    return true;
  },

  prefObserver : {

    getService : function() {
      return Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranchInternal);
    },

    register: function() {
      this.getService().addObserver('extensions.highlander', this, false);
    },

    unregister: function() {
      this.getService().removeObserver('extensions.highlander', this);
    },

    observe : function(subject, topic, data) {
      if (topic != 'nsPref:changed') {
        return;
      }

      if (data == 'extensions.highlander.whitelist') {
        window.HighlanderWhitelist.load();
      }
    }

  }
};

window.addEventListener('unload', function() { HighlanderWhitelist.prefObserver.unregister(); }, false);
